// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef DDNDCOMMON_H
#define DDNDCOMMON_H

#define DND_INTERFACE "com.deepin.dtk.FileDrag"
#define DND_OBJPATH "/Ddnd"
#define DND_MIME_SERVICE "x-dtk-file-drag/service"
#define DND_MIME_PID "x-dtk-file-drag/pid"
#define DND_MIME_UUID "x-dtk-file-drag/uuid"
#define DND_TARGET_URL_KEY "targetUrl"

#endif // DDNDCOMMON_H
